//-----------------------------------------------------------------------------
// GenFilter.h - Filter Generation Class
// $Version 1.8 (04/05/03)
// Copyright (C) 2000-2003 by Yasuyuki Kashiwagi. All Rights Reserved.
//-----------------------------------------------------------------------------

#if !defined(__GENFILTER_H)
#define      __GENFILTER_H

#if !defined(__PBPLUGIN_H)
#include "../../../../PbPlugin.h"
#endif


///////////////////////////////////////////////////////////////////////////////
// PBF_Execute pf[^ (V0݊)

struct ExecuteArg 
{
public:
	HWND                 hwnd;
	const DWORD*         pSrc;
	DWORD*               pDst;
	const Pixel32*       src;
	Pixel32*             dst;
	BYTE*                mask;
	RECT*                area;
	DWORD*               maskflag;
	int                  width;
	int                  height;
	long                 lData;
	PBF_ProgressCallback pProgress;
	PFilterImage*        pImage;
	PProvidedFunc*       pFunc;

public:
	ExecuteArg() 
	{
	}

	ExecuteArg(HWND hWnd, PFilterImage* pImage, PProvidedFunc* pFunc)
	{
		this->hwnd      = hWnd;
		this->pSrc      = pImage->pSrc;
		this->pDst      = pImage->pDst;
		this->src       = (const Pixel32*)pImage->pSrc;
		this->dst       = (Pixel32*)pImage->pDst;
		this->mask      = pImage->pMask;
		this->area      = &pImage->rcSelection;
		this->maskflag  = &pImage->dwMaskFlag;
		this->width     = pImage->nWidth;
		this->height    = pImage->nHeight;
		this->pProgress = NULL;
		this->lData     = 0;
		this->pImage    = pImage;
		this->pFunc     = pFunc;
	}
};


///////////////////////////////////////////////////////////////////////////////
// PBF_Setup pf[^ (V0݊)

struct SetupArg
{
	HWND            hwnd;
	const DWORD*    pSrc;
	const Pixel32*  src;
	int             width;
	int             height;
	PFilterImage*   pImage;
	PProvidedFunc*  pFunc;

	SetupArg()
	{
	}

	SetupArg(HWND hWnd, PFilterImage* pImage, PProvidedFunc* pFunc)
	{
		this->hwnd    = hWnd;
		this->pSrc    = pImage->pSrc;
		this->src     = (const Pixel32*)pImage->pSrc;
		this->width   = pImage->nWidth;
		this->height  = pImage->nHeight;
		this->pImage  = pImage;
		this->pFunc   = pFunc;
	}
};


///////////////////////////////////////////////////////////////////////////////
// tB^x[XNX(̃NXhĎg܂)

class FilterBase 
{
	char  m_szName[127];
	char  m_szKind[127];
	int   m_nProgressMax;
	BOOL  m_bInitProgressProc;
	DWORD m_dwFormat;

protected:
	void SetSupportedFormat(DWORD dwFormat)
	{
		m_dwFormat = dwFormat;
	}

	void SetName(const char* name) 
	{
		strncpy(m_szName, name, sizeof(m_szName));
	}

	void SetKind(const char* kind) 
	{
		strncpy(m_szKind, kind, sizeof(m_szKind));
	}

public:
	FilterBase() 
	{
		SetName("Filter Name|tB^̖O");
		SetKind("Filter Kind|tB^̎");
		SetSupportedFormat(PBF_TYPE_FULLCOLOR);
		m_bInitProgressProc = FALSE;
	}

	virtual ~FilterBase() 
	{
	}

	DWORD GetSupportedFormat() const
	{
		return m_dwFormat;
	}

	const char* GetName() const 
	{ 
		return m_szName; 
	}

	const char* GetKind() const 
	{ 
		return m_szKind; 
	}

	virtual void Execute(ExecuteArg* pArg) = 0;

	virtual BOOL Setup(SetupArg* pArg) 
	{ 
		(void)pArg;
		return TRUE; 
	}

	BOOL QueryClientInfo(DWORD dwQueryNumber, void* pResult)
	{
		// PictBear Second Edition ɃNGs
		PBF_QueryClientInfoProc pQueryProc;

		const char* lpszProc = "PBF_QueryClientInfo";
		char        szClientEXE[MAX_PATH];
		HINSTANCE   hModule;
		BOOL        bResult = FALSE;

		GetModuleFileName(NULL, szClientEXE, MAX_PATH);

		if ((hModule = LoadLibrary(szClientEXE)) != NULL) 
		{
			pQueryProc = (PBF_QueryClientInfoProc)GetProcAddress(hModule, lpszProc);
			if (pQueryProc != NULL)
				bResult = pQueryProc(dwQueryNumber, pResult);
			FreeLibrary(hModule);
		}
		
		return bResult;
	}

	void ProgressStart(ExecuteArg* pArg, int nRange) 
	{
		if (!m_bInitProgressProc) 
		{
			if (pArg->pFunc->ProgressStart != NULL) 
			{
				pArg->pFunc->ProgressStart(0, nRange);
				m_nProgressMax = nRange;
				m_bInitProgressProc = TRUE;
			}
		}
	}

	void ProgressEnd(ExecuteArg* pArg) 
	{
		if (m_bInitProgressProc) 
		{
			pArg->pFunc->ProgressEnd();
			m_bInitProgressProc = FALSE;
		}
	}

	void ProgressSetPos(ExecuteArg* pArg, int nPos) 
	{
		if (m_bInitProgressProc)
		{
			if (nPos>=0 && nPos<=m_nProgressMax)
				pArg->pFunc->ProgressSetPos(nPos);
		}
	}
};


///////////////////////////////////////////////////////////////////////////////
// tB^vOC}l[W(gpȂŉ)

#define MAX_FILTER 256

class FilterPluginManager 
{
	int         m_nCount;
	char        m_szCopyright[127];
	FilterBase* m_pFilters[MAX_FILTER];

public:
	FilterPluginManager() 
	{
		m_nCount = 0;
		SetCopyright("Powered by GenFilter.h - Copyright (C) 2000-2004 by Yasuyuki Kashiwagi.");
	}

	virtual ~FilterPluginManager() 
	{
		for (int i=0; i<m_nCount; i++)
			delete m_pFilters[i];
	}

	BOOL AddFilter(FilterBase* filter) 
	{
		if (m_nCount >= MAX_FILTER) 
			return FALSE;

		m_pFilters[m_nCount++] = filter;

		return TRUE;
	}

	void Execute(int no, ExecuteArg* arg) 
	{
		m_pFilters[no]->Execute(arg);
	}

	BOOL Setup(int no, SetupArg* arg) 
	{
		return m_pFilters[no]->Setup(arg);
	}

	int GetCount() const 
	{ 
		return m_nCount; 
	}

	DWORD GetSupportedFormat(int no) const 
	{ 
		return m_pFilters[no]->GetSupportedFormat(); 
	}

	const char* GetKind(int no) const 
	{ 
		return m_pFilters[no]->GetKind(); 
	}
	
	const char* GetName(int no) const 
	{ 
		return m_pFilters[no]->GetName(); 
	}
	
	const char* GetCopyright() const 
	{ 
		return m_szCopyright; 
	}

	void SetCopyright(const char* copyright) 
	{ 
		strncpy(m_szCopyright, copyright, sizeof(m_szCopyright)); 
	}
};


///////////////////////////////////////////////////////////////////////////////
// }N

#define DLLExport extern "C" __declspec (dllexport)

// tB^}bvJn
#define BEGIN_FILTER_MAP()			\
	FilterPluginManager g_Manager;	\
	static void __AddFilters() {
#

// tB^`
#define DECLARE_FILTER(FilterClass)			\
	g_Manager.AddFilter(new FilterClass);
#

// 쌠`
#define DECLARE_COPYRIGHT(copyright)	\
	g_Manager.SetCopyright(copyright);	
#

// tB^}bvI
#define END_FILTER_MAP()	\
}							\
\
PBF_QueryClientInfoProc g_QueryClientInfoProc;					\
HINSTANCE g_hInstance;											\
extern "C" BOOL WINAPI DllMain(									\
	HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved) {	\
	(void)lpvReserved;											\
	if (fdwReason == DLL_PROCESS_ATTACH) { 						\
		g_hInstance = hinstDLL;									\
		__AddFilters();											\
	}															\
	return TRUE;												\
}																\
\
DLLExport WORD WINAPI PB_GetPluginInfo(LPSTR buf, int buflen) {	\
	strncpy(buf, g_Manager.GetCopyright(), buflen);				\
	return MAKEWORD(PBPLUGIN_FILTER, PBPLUGIN_FILTER_VERSION);	\
}																\
\
DLLExport int WINAPI PBF_FilterCount(void) {\
	return g_Manager.GetCount();			\
}											\
\
DLLExport DWORD WINAPI PBF_GetSupportedFormat(int nIndex) {		\
	return g_Manager.GetSupportedFormat(nIndex);				\
}																\
\
DLLExport void WINAPI PBF_GetName(									\
	int no, LPSTR kind, int kind_len, LPSTR name, int name_len) {	\
	strncpy(kind, g_Manager.GetKind(no), kind_len);					\
	strncpy(name, g_Manager.GetName(no), name_len);					\
}																	\
\
DLLExport void WINAPI PBF_Execute(					\
	int no, HWND hwnd,								\
	PFilterImage* pImage, PProvidedFunc* pFunc) { 	\
	ExecuteArg arg(hwnd, pImage, pFunc);			\
	g_Manager.Execute(no, &arg);					\
}													\
\
DLLExport BOOL WINAPI PBF_Setup(					\
	int no, HWND hwnd,								\
	PFilterImage* pImage, PProvidedFunc* pFunc) { 	\
	SetupArg arg(hwnd, pImage, pFunc);				\
	return g_Manager.Setup(no, &arg);				\
}
#


#ifdef _MSC_VER
#pragma comment(linker, "/EXPORT:PB_GetPluginInfo=_PB_GetPluginInfo@8")
#pragma comment(linker, "/EXPORT:PBF_FilterCount=_PBF_FilterCount@0")
#pragma comment(linker, "/EXPORT:PBF_GetName=_PBF_GetName@20")
#pragma comment(linker, "/EXPORT:PBF_GetSupportedFormat=_PBF_GetSupportedFormat@4")
#pragma comment(linker, "/EXPORT:PBF_Setup=_PBF_Setup@16")
#pragma comment(linker, "/EXPORT:PBF_Execute=_PBF_Execute@16")
#endif


#endif // __GENFILTER_H

