//-----------------------------------------------------------------------------
// Mosaic Filters
// Copyright (C) 2000-2003 by Yasuyuki Kashiwagi. All Rights Reserved.
//-----------------------------------------------------------------------------

#include "Mosaic.h"

MosaicFilter::MosaicFilter()
{
	SetSupportedFormat(PBF_TYPE_FULLCOLOR);
	SetKind("Special|");
	SetName("Mosaic|UCN");
}

void MosaicFilter::Execute(ExecuteArg* arg)
{
	const int size = 8;
	RECT rec = *(arg->area);

	ProgressStart(arg, rec.bottom - rec.top);
	for (int y=rec.top; y<rec.bottom-size; y+=size) 
	{
		for (int x=rec.left; x<rec.right-size; x+=size) 
		{
			int r, g, b, a;
			int mx, my;
			int div;

			r = g = b = a = div = 0;

			for (my=y; my<y+size && my<rec.bottom; my++) 
			{
				for (mx=x; mx<x+size && mx<rec.right; mx++) 
				{
					Pixel32 pixel = arg->src[mx + my*arg->width];
					r += pixel.item.r;
					g += pixel.item.g;
					b += pixel.item.b;
					a += pixel.item.a;
					++div;
				}
			}

			r /= div;
			g /= div;
			b /= div;
			a /= div;

			for (my=y; my<y+size && my<rec.bottom; my++) 
			{
				for (mx=x; mx<x+size && mx<rec.right; mx++) 
				{
					int pos       = mx + my*arg->width;
					Pixel32* dstp = &(arg->dst[pos]);

					dstp->item.r = (BYTE)r;
					dstp->item.g = (BYTE)g;
					dstp->item.b = (BYTE)b;
					dstp->item.a = (BYTE)a;
				}
			}
		}
		ProgressSetPos(arg, y - rec.top);
	}
	ProgressEnd(arg);
}	
